<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WSSV_Ajax.
 *
 * @class	Iconic_WSSV_Ajax
 * @version  1.0.0
 * @author   Iconic
 */
class Iconic_WSSV_Ajax {

	/**
	 * Instance.
	 */
	private static $instance;

	/**
	 * Init.
	 */
	public static function init() {
		if ( ! isset( self::$instance ) ) {
			self::$instance = new Iconic_WSSV_Ajax;
			self::$instance->add_ajax_events();
		}
	}

	/**
	 * Hook in methods.
	 */
	private static function add_ajax_events() {
		$ajax_events = array(
			'get_variations_count' => false,
			'get_combined_count' => false,
			'get_products_count' => false,
			'process_variation_visibility' => false,
			'process_variation_sort_order' => false
		);

		foreach ( $ajax_events as $ajax_event => $nopriv ) {
			add_action( 'wp_ajax_iconic_woo_show_single_variations_' . $ajax_event, array( __CLASS__, $ajax_event ) );

			if ( $nopriv ) {
				add_action( 'wp_ajax_nopriv_iconic_woo_show_single_variations_' . $ajax_event, array( __CLASS__, $ajax_event ) );
			}
		}
	}

	/**
	 * Get variations count.
	 */
	public static function get_variations_count( $return = false ) {
		$count = (array) wp_count_posts( 'product_variation' );
		$count = array_sum( array_map('intval', $count ) );

		if( $return ) {
			return $count;
		}

		$response = array(
			'success' => true,
			'count' => $count
		);

		wp_send_json( $response );
	}

	/**
	 * Get products count.
	 */
	public static function get_products_count( $return = false ) {
		$count = (array) wp_count_posts( 'product' );
		$count = array_sum( array_map('intval', $count ) );

		if( $return ) {
			return $count;
		}

		$response = array(
			'success' => true,
			'count' => $count
		);

		wp_send_json( $response );
	}

	/**
	 * Get products count.
	 */
	public static function get_combined_count() {
		$variations_count = self::get_variations_count( true );
		$products_count = self::get_products_count( true );

		$response = array(
			'success' => true,
			'count' => $variations_count+$products_count
		);

		wp_send_json( $response );
	}

	/**
	 * Process variation visibility.
	 */
	public static function process_variation_visibility() {
		$args = array(
			'post_type' => 'product_variation',
			'posts_per_page' => $_POST['limit'],
			'offset' => $_POST['offset'],
			'post_status' => 'any'
		);

		$variations = new WP_Query( $args );

		if ( $variations->have_posts() ) {

			while ( $variations->have_posts() ) {
				$variations->the_post();

				$visibility = Iconic_WSSV_Product_Variation::get_visibility( get_the_id() );
				Iconic_WSSV_Product_Variation::set_visibility( get_the_id(), $visibility );
			}

		}

		wp_reset_postdata();

		wp_send_json( array( 'success' => true ) );
	}

	/**
	 * Process variation sort_order.
	 */
	public static function process_variation_sort_order() {
		$args = array(
			'post_type' => array( 'product', 'product_variation' ),
			'posts_per_page' => $_POST['limit'],
			'offset' => $_POST['offset'],
			'post_status' => 'any'
		);

		$products = new WP_Query( $args );

		if ( $products->have_posts() ) {

			while ( $products->have_posts() ) {
				$products->the_post();

				$product_id = get_the_id();

				if( get_post_type() === "product" ) {
					$menu_order = get_post_field( 'menu_order', $product_id );
					$custom_menu_order = self::format_menu_order( $menu_order, 0 );

					update_post_meta( $product_id, 'iconic_woo_show_single_variations_menu_order', $custom_menu_order );

					do_action( 'iconic_updated_product_sort_order', $product_id, $custom_menu_order );
				} else {
					$parent_id = wp_get_post_parent_id( $product_id );
					$menu_order = get_post_field( 'menu_order', $product_id );
					$parent_menu_order = get_post_field( 'menu_order', $parent_id );
					$custom_menu_order = self::format_menu_order( $parent_menu_order, $menu_order+1 );

					update_post_meta( $product_id, 'iconic_woo_show_single_variations_menu_order', $custom_menu_order );

					do_action( 'iconic_updated_variation_sort_order', $product_id, $custom_menu_order, $parent_id );
				}

				do_action( 'iconic_updated_sort_order', $product_id, $custom_menu_order );
			}
		}

		wp_reset_postdata();

		wp_send_json( array( 'success' => true ) );
	}

	/**
	 * Format menu order
	 *
	 * @param int $menu_order
	 * @param int $sub_menu_order
	 * @return int
	 */
	public static function format_menu_order( $menu_order, $sub_menu_order = 0 ) {
		return absint( sprintf( '%d%04d', $menu_order, $sub_menu_order ) );
	}
}