<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WSSV_Product_Variation.
 *
 * @class	Iconic_WSSV_Product_Variation
 * @version  1.0.0
 * @author   Iconic
 */
class Iconic_WSSV_Product_Variation {

	/**
	 * Set visibility.
	 *
	 * @param int $variation_id
	 * @param array $visibility
	 */
	public static function set_visibility( $variation_id, $visibility ) {
		update_post_meta( $variation_id, '_visibility', $visibility );

		if ( version_compare( WC_VERSION, '3.0.0', '>=' ) ) {
			$terms = array();
			$visibility = implode( '-', $visibility );

			switch ( $visibility ) {
				case 'catalog-filtered' :
					$terms[] = "exclude-from-search";
					break;
				case 'catalog-search' :
					$terms[] = "exclude-from-filtered";
					break;
				case 'catalog' :
					$terms[] = "exclude-from-search";
					$terms[] = "exclude-from-filtered";
					break;
				case 'filtered-search' :
					$terms[] = "exclude-from-catalog";
					break;
				case 'search' :
					$terms[] = "exclude-from-catalog";
					$terms[] = "exclude-from-filtered";
					break;
				case 'filtered' :
					$terms[] = "exclude-from-catalog";
					$terms[] = "exclude-from-search";
					break;
				case 'hidden' :
					$terms[] = "exclude-from-catalog";
					$terms[] = "exclude-from-search";
					$terms[] = "exclude-from-filtered";
					break;
			}

			if( empty( $visibility ) ) {
				$terms[] = "exclude-from-catalog";
				$terms[] = "exclude-from-search";
				$terms[] = "exclude-from-filtered";
			}

			if ( ! is_wp_error( wp_set_post_terms( $variation_id, $terms, 'product_visibility', false ) ) ) {
				delete_transient( 'wc_featured_products' );
				do_action( 'woocommerce_product_set_visibility', $variation_id, $visibility );
			}
		}
	}

	/**
	 * Get visibility.
	 *
	 * @param int $variation_id
	 * @return string
	 */
	public static function get_visibility( $variation_id ) {
		$visibility = get_post_meta( $variation_id, '_visibility', true );

		if( ! is_array( $visibility ) ) {
			return array( "hidden" );
		}

		sort( $visibility );

		return $visibility;
	}
}